<?php

namespace Pinpoint\ProductPersonalisation\Block\Adminhtml\ImageTemplate\Edit;

use Magento\Framework\View\Element\Template;

/**
 * Class ImageTemplateRows
 * @package Pinpoint\ProductPersonalisation\Block\Adminhtml\ImageTemplate\Edit
 */
class ImageTemplateRows extends Template
{
    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageTemplate
     */
    protected $imageTemplate;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageRow
     */
    protected $imageRow;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\Font
     */
    protected $fonts;

    /**
     * ImageTemplateRows constructor.
     * @param Template\Context $context
     * @param \Pinpoint\ProductPersonalisation\Model\ImageTemplate $imageTemplate
     * @param \Pinpoint\ProductPersonalisation\Model\ImageRow $imageRow
     * @param \Pinpoint\ProductPersonalisation\Model\Font $font
     * @param array $data
     */
    public function __construct(
        Template\Context $context,
        \Pinpoint\ProductPersonalisation\Model\ImageTemplate $imageTemplate,
        \Pinpoint\ProductPersonalisation\Model\ImageRow $imageRow,
        \Pinpoint\ProductPersonalisation\Model\Font $font,
        array $data = []
    ) {
        $this->imageTemplate = $imageTemplate;
        $this->imageRow = $imageRow;
        $this->fonts = $font;

        parent::__construct($context, $data);
    }

    /**
     * @return \Pinpoint\ProductPersonalisation\Model\ImageTemplate|null
     */
    public function getImageTemplate()
    {
        if ($templateId = $this->getRequest()->getParam('id')) {
            $template = $this->imageTemplate->load($templateId);

            return $template;
        }

        return null;
    }

    /**
     * @return \Magento\Framework\DataObject[]|null
     */
    public function getTemplateRows()
    {
        if ($imageTemplate = $this->getImageTemplate()) {
            return $this->imageRow
                ->getCollection()
                ->addFieldToFilter('parent_template_id', $imageTemplate->getId())
                ->getItems()
            ;
        }

        return null;
    }

    /**
     * @param $index
     * @return int
     */
    public function getTemplateRowArrayKey($index) {
        if ($rows = $this->getTemplateRows()) {
            return (int)array_keys($rows)[$index];
        }
    }

    /**
     * @return \Magento\Framework\DataObject[]
     */
    public function getFonts()
    {
        return $this->fonts->getCollection()->getItems();
    }

}