<?php

namespace Pinpoint\ProductPersonalisation\Block\Product\View\Options\Type;

class Select extends \Magento\Catalog\Block\Product\View\Options\Type\Select
{
    protected $blankProductHelper;

    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Framework\Pricing\Helper\Data $pricingHelper,
        \Magento\Catalog\Helper\Data $catalogData,
        \Pinpoint\BlankProduct\Helper\Quantity $quantityHelper,
        array $data = []
    ) {
        $this->blankProductHelper = $quantityHelper;
        parent::__construct($context, $pricingHelper, $catalogData, $data);
    }

    /**
     * Return html for control element
     *
     * @return string
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    public function getValuesHtml()
    {
        $_option = $this->getOption();
        $configValue = $this->getProduct()->getPreconfiguredValues()->getData('options/' . $_option->getId());
        $store = $this->getProduct()->getStore();

        $this->setSkipJsReloadPrice(1);
        // Remove inline prototype onclick and onchange events

        if ($_option->getType() == \Magento\Catalog\Api\Data\ProductCustomOptionInterface::OPTION_TYPE_DROP_DOWN ||
            $_option->getType() == \Magento\Catalog\Api\Data\ProductCustomOptionInterface::OPTION_TYPE_MULTIPLE
        ) {
            $require = $_option->getIsRequire() ? ' required' : '';
            $extraParams = '';
            $select = $this->getLayout()->createBlock(
                \Magento\Framework\View\Element\Html\Select::class
            )->setData(
                [
                    'id' => 'select_' . $_option->getId(),
                    'class' => $require . ' product-custom-option admin__control-select' . ' ' . str_replace(' ', '-', strtolower($_option->getData('title')))
                ]
            );
            if ($_option->getType() == \Magento\Catalog\Api\Data\ProductCustomOptionInterface::OPTION_TYPE_DROP_DOWN) {
              if ($_option->getTitle() == 'Choose Font') {
                  $select->setName('options[' . $_option->getid() . ']')->addOption('', __('-- Choose Font --'));
              } else {
                  $select->setName('options[' . $_option->getid() . ']')->addOption('', __('-- Please Select --'));
              }
            } else {
                $select->setName('options[' . $_option->getid() . '][]');
                $select->setClass('multiselect admin__control-multiselect' . $require . ' product-custom-option');
            }
            foreach ($_option->getValues() as $_value) {
                $priceStr = $this->_formatPrice(
                    [
                        'is_percent' => $_value->getPriceType() == 'percent',
                        'pricing_value' => $_value->getPrice($_value->getPriceType() == 'percent'),
                    ],
                    false
                );

                /**
                 * Blank product functionality. If blank product sku being uses, check stock.
                 * If not blank product stock, check stock against the main product.
                 * Disable dropdown option if fails stock check
                 */
                $optionAttributes = ['price' => $this->pricingHelper->currencyByStore($_value->getPrice(true), $store, false)];
                if ($sku = $_value->getSku()) {
                    if (preg_match('/(?<=\#)(.*?)(?=\#)/', $sku, $blankSku)) {
                        if (!$this->blankProductHelper->getStockStatus($blankSku[0])) {
                            $optionAttributes['disabled'] = 'disabled';
                        }
                    } else {
                        if (!$this->blankProductHelper->getStockStatus($this->getProduct()->getSku())) {
                            $optionAttributes['disabled'] = 'disabled';
                        }
                    }
                }

                $select->addOption(
                    $_value->getOptionTypeId(),
                    $_value->getTitle() . ' ' . strip_tags($priceStr) . '',
                    $optionAttributes
                );
            }
            if ($_option->getType() == \Magento\Catalog\Api\Data\ProductCustomOptionInterface::OPTION_TYPE_MULTIPLE) {
                $extraParams = ' multiple="multiple"';
            }
            if (!$this->getSkipJsReloadPrice()) {
                $extraParams .= ' onchange="opConfig.reloadPrice()"';
            }
            $extraParams .= ' data-selector="' . $select->getName() . '"';
            $select->setExtraParams($extraParams);

            if ($_option->getData('is_copy')) {
                $select->setExtraParams(sprintf('data-postLabel="%s"', $_option->getTitle()));
            }

            if ($configValue) {
                $select->setValue($configValue);
            }

            return $select->getHtml();
        }

        if ($_option->getType() == \Magento\Catalog\Api\Data\ProductCustomOptionInterface::OPTION_TYPE_RADIO ||
            $_option->getType() == \Magento\Catalog\Api\Data\ProductCustomOptionInterface::OPTION_TYPE_CHECKBOX
        ) {
            $selectHtml = '<div class="options-list nested" id="options-' . $_option->getId() . '-list">';
            $require = $_option->getIsRequire() ? ' required' : '';
            $arraySign = '';
            switch ($_option->getType()) {
                case \Magento\Catalog\Api\Data\ProductCustomOptionInterface::OPTION_TYPE_RADIO:
                    $type = 'radio';
                    $class = 'radio admin__control-radio';
//                    if (!$_option->getIsRequire()) {
//                        $selectHtml .= '<div class="field choice admin__field admin__field-option">' .
//                            '<input type="radio" id="options_' .
//                            $_option->getId() .
//                            '" class="' .
//                            $class .
//                            ' product-custom-option" name="options[' .
//                            $_option->getId() .
//                            ']"' .
//                            ' data-selector="options[' . $_option->getId() . ']"' .
//                            ($this->getSkipJsReloadPrice() ? '' : ' onclick="opConfig.reloadPrice()"') .
//                            ' value="" checked="checked" /><label class="label admin__field-label" for="options_' .
//                            $_option->getId() .
//                            '"><span>' .
//                            __('None') . '</span></label></div>';
//                    }
                    break;
                case \Magento\Catalog\Api\Data\ProductCustomOptionInterface::OPTION_TYPE_CHECKBOX:
                    $type = 'checkbox';
                    $class = 'checkbox admin__control-checkbox';
                    $arraySign = '[]';
                    break;
            }
            $count = 1;
            $arImages = [];
            foreach ($_option->getValues() as $_value) {
                $count++;

                $priceStr = $this->_formatPrice(
                    [
                        'is_percent' => $_value->getPriceType() == 'percent',
                        'pricing_value' => $_value->getPrice($_value->getPriceType() == 'percent'),
                    ]
                );

                $htmlValue = $_value->getOptionTypeId();
                if ($arraySign) {
                    $checked = is_array($configValue) && in_array($htmlValue, $configValue) ? 'checked' : '';
                } else {
                    $checked = $configValue == $htmlValue ? 'checked' : '';
                }

                $dataSelector = 'options[' . $_option->getId() . ']';
                if ($arraySign) {
                    $dataSelector .= '[' . $htmlValue . ']';
                }

                if ($_option->getTitle() === 'Recommended Options' &&
                    $_value->getTitle() === 'Select Any Additional Options'
                ) {
                    $class = 'radio admin__control-radio  product-custom-option';
                    $selectHtml .= '<div class="field field__label">Select Any Additional Options</div>
                        <div class="field choice admin__field admin__field-option">
                        <input id="noselect" type="radio" name="options['. $_option->getId() .']" class="' . $class . '" value="" data-selector="' . $dataSelector . '" price="0.00" checked="checked" />
                        <div class="check"><div class="inside"></div></div><label class="label admin__field-label" for="noselect"><span>None</span></span></label></div>';
                    continue;
                }

                $selectHtml .= '<div class="field choice admin__field admin__field-option' .
                    $require .
                    '">' .
                    '<input type="' .
                    $type .
                    '" class="' .
                    $class .
                    ' ' .
                    $require .
                    ' product-custom-option"' .
                    ($this->getSkipJsReloadPrice() ? '' : ' onclick="opConfig.reloadPrice()"') .
                    ' name="options[' .
                    $_option->getId() .
                    ']' .
                    $arraySign .
                    '" id="options_' .
                    $_option->getId() .
                    '_' .
                    $count .
                    '" value="' .
                    $htmlValue .
                    '" ' .
                    $checked .
                    ' data-selector="' . $dataSelector . '"' .
                    ' price="' .
                    $this->pricingHelper->currencyByStore($_value->getPrice(true), $store, false) .
                    '" />' .
                    '<div class="check"><div class="inside"></div></div><label class="label admin__field-label" for="options_' .
                    $_option->getId() .
                    '_' .
                    $count .
                    '"><span>' .
                    $_value->getTitle() .
                    '</span> ' .
                    $priceStr .
                    '</label>';
                $selectHtml .= '</div>';

                $arImages[] = str_replace(['(', ')'], '_', str_replace(' ', '-', strtolower($_value->getTitle())));
            }
            $selectHtml .= '</div>';

            $selectHtml .= '<div class="option-images">';
            foreach ($arImages as $image) {
                $selectHtml .= '<div class="option-image">';

                $selectHtml .= '<img src="/media/wysiwyg/bags/'. $image .'.jpg" />';

                $selectHtml .= '</div>';
            }
            $selectHtml .= '</div>';

            return $selectHtml;
        }
    }
}
