<?php
namespace Pinpoint\ProductPersonalisation\Controller\Adminhtml\Font;

use Magento\Backend\App\Action;
use Pinpoint\ProductPersonalisation\Model\Page;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Exception\LocalizedException;

/**
 * Class Save
 * @package Pinpoint\ProductPersonalisation\Controller\Adminhtml\Font
 */
class Save extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Pinpoint_ProductPersonalisation::fontAddEdit';

    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\Font
     */
    protected $font;

    /**
     * Save constructor.
     * @param Action\Context $context
     * @param DataPersistorInterface $dataPersistor
     * @param \Pinpoint\ProductPersonalisation\Model\Font $font
     */
    public function __construct(
        Action\Context $context,
        DataPersistorInterface $dataPersistor,
        \Pinpoint\ProductPersonalisation\Model\Font $font
    ) {
        $this->dataPersistor = $dataPersistor;
        $this->font = $font;

        parent::__construct($context);
    }

    /**
     * Save action
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue();
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        if ($data) {
            if (empty($data['id'])) {
                $data['id'] = null;
            }

            /** @var \Pinpoint\ProductPersonalisation\Model\Font $model */
            $model = $this->font;

            $id = $this->getRequest()->getParam('id');
            if ($id) {
                $model->load($id);
            }

            if (isset($data['font_file_upload']) && is_array($data['font_file_upload'])) {
                $data['font_file'] = str_replace(' ', '_', $data['font_file_upload'][0]['name']);
                unset($data['font_file_upload']);
            }

            $model->setData($data);

            try {
                $model->save();
                $this->messageManager->addSuccess(__('%1 has been saved.', $data['font_name']));
                $this->dataPersistor->clear('pinpoint_productpersonalisation_font');
                if ($this->getRequest()->getParam('back')) {
                    return $resultRedirect->setPath('*/*/edit', ['id' => $model->getId(), '_current' => true]);
                }
                return $resultRedirect->setPath('*/fonts/fontlist');
            } catch (LocalizedException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addException($e, __('Something went wrong while saving the data.'));
            }

            $this->dataPersistor->set('pinpoint_productpersonalisation_font', $data);
            return $resultRedirect->setPath('*/*/edit', ['id' => $this->getRequest()->getParam('id')]);
        }

        return $resultRedirect->setPath('*/fonts/fontlist');
    }    
}
