<?php

namespace Pinpoint\ProductPersonalisation\Controller\Adminhtml\Font;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Filesystem\DirectoryList;


class Upload extends Action
{
    /**
     * @var \Magento\Framework\Filesystem
     */
    protected $_filesystem;

    /**
     * @var \Magento\MediaStorage\Model\File\Uploader
     */
    protected $uploader;

    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * Upload constructor.
     * @param Context $context
     * @param \Magento\Framework\Filesystem $fileSystem
     * @param \Magento\MediaStorage\Model\File\UploaderFactory $uploader
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     */
    public function __construct(
        Context $context,
        \Magento\Framework\Filesystem $fileSystem,
        \Magento\MediaStorage\Model\File\UploaderFactory $uploader,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager
    ) {
        $this->_filesystem = $fileSystem;
        $this->uploader = $uploader;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->storeManager = $storeManager;

        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        /** @var \Magento\Framework\Controller\Result\Json $resultPage */
        $resultPage = $this->resultJsonFactory->create();

        if ($_FILES['font_file_upload']['name']) {
            try {
                $uploader = $this->uploader->create(['fileId' => 'font_file_upload']);
                $uploader->setAllowedExtensions(['ttf', 'otf']);
                $uploader->setAllowRenameFiles(false);
                $uploader->setFilesDispersion(false);
                $mediaDirectory = $this->_filesystem->getDirectoryRead(DirectoryList::MEDIA);
                $savePath = $mediaDirectory->getAbsolutePath() . 'pinpoint_fonts' . DIRECTORY_SEPARATOR;
                $save = $uploader->save($savePath, $_FILES['font_file_upload']['name']);

                $save['file'] = DIRECTORY_SEPARATOR . 'pub' . DIRECTORY_SEPARATOR . 'media' . DIRECTORY_SEPARATOR . 'pinpoint_fonts' . $save['file'];
                $save['url'] = $save['file'];

                $save['cookie'] = [
                    'name' => $this->_getSession()->getName(),
                    'value' => $this->_getSession()->getSessionId(),
                    'lifetime' => $this->_getSession()->getCookieLifetime(),
                    'path' => $this->_getSession()->getCookiePath(),
                    'domain' => $this->_getSession()->getCookieDomain(),
                ];
            } catch (Exception $e) {
                \Zend_Debug::dump($e->getMessage());
            }
        }

        return $resultPage->setData($save);
    }
}
