<?php

namespace Pinpoint\ProductPersonalisation\Controller\Adminhtml\ImageTemplate;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Filesystem\DirectoryList;

/**
 * Class ImageUpload
 * @package Pinpoint\ProductPersonalisation\Controller\Adminhtml\ImageTemplate
 */
class ImageUpload extends Action
{
    /**
     * @var \Magento\Framework\Filesystem
     */
    protected $_filesystem;

    /**
     * @var \Magento\MediaStorage\Model\File\Uploader
     */
    protected $uploader;

    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * ImageUpload constructor.
     * @param Context $context
     * @param \Magento\Framework\Filesystem $fileSystem
     * @param \Magento\MediaStorage\Model\File\UploaderFactory $uploader
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     */
    public function __construct(
        Context $context,
        \Magento\Framework\Filesystem $fileSystem,
        \Magento\MediaStorage\Model\File\UploaderFactory $uploader,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager
    ) {
        $this->_filesystem = $fileSystem;
        $this->uploader = $uploader;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->storeManager = $storeManager;

        parent::__construct($context);
    }

    /**
     * Check the permission to run it
     *
     * @return boolean
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Pinpoint_ProductPersonalisation::imagetemplate');
    }

    /**
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        /** @var \Magento\Framework\Controller\Result\Json $resultPage */
        $resultPage = $this->resultJsonFactory->create();
        if ($_FILES['template_image']['name']) {
            try {
                $uploader = $this->uploader->create(['fileId' => 'template_image']);
                $uploader->setAllowedExtensions(['jpg', 'jpeg', 'gif', 'png']);
                $uploader->setAllowRenameFiles(true);
                $uploader->setFilesDispersion(true);
                // get media directory
                $mediaDirectory = $this->_filesystem->getDirectoryRead(DirectoryList::MEDIA);
                // save the image to media directory
                $savePath = $mediaDirectory->getAbsolutePath() . 'pinpoint_pp_images' . DIRECTORY_SEPARATOR;
                $save = $uploader->save($savePath, md5(rand() . time() . uniqid()) . "." . $uploader->getFileExtension());

                $save['file'] = DIRECTORY_SEPARATOR . 'pub' . DIRECTORY_SEPARATOR . 'media' . DIRECTORY_SEPARATOR . 'pinpoint_pp_images' . $save['file'];
                $save['url'] = $save['file'];

                $mediaUrl = $this->storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA);
                if (strpos($mediaUrl, '/pub/') === false) {
                    $save['url'] = str_replace('pub/', '', $save['url']);
                }

                $save['cookie'] = [
                    'name' => $this->_getSession()->getName(),
                    'value' => $this->_getSession()->getSessionId(),
                    'lifetime' => $this->_getSession()->getCookieLifetime(),
                    'path' => $this->_getSession()->getCookiePath(),
                    'domain' => $this->_getSession()->getCookieDomain(),
                ];
            } catch (Exception $e) {
                \Zend_Debug::dump($e->getMessage());
            }
        }
        return $resultPage->setData($save);
    }

    /**
     * @return bool
     */
    public function _processUrlKeys()
    {
        return true;
    }
}
