<?php
namespace Pinpoint\ProductPersonalisation\Controller\Adminhtml\ImageTemplate;

use Magento\Backend\App\Action;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Exception\LocalizedException;

/**
 * Class Save
 * @package Pinpoint\ProductPersonalisation\Controller\Adminhtml\ImageTemplate
 */
class Save extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Pinpoint_ProductPersonalisation::templateForm';

    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageTemplate
     */
    protected $imageTemplate;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageRow
     */
    protected $imageTemplateRow;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageRowRepository
     */
    protected $imageTemplateRowRepository;

    /**
     * Save constructor.
     * @param Action\Context $context
     * @param DataPersistorInterface $dataPersistor
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Pinpoint\ProductPersonalisation\Model\ImageTemplate $imageTemplate
     * @param \Pinpoint\ProductPersonalisation\Model\ImageRow $imageTemplateRow
     * @param \Pinpoint\ProductPersonalisation\Model\ImageRowRepository $imageTemplateRowRepository
     */
    public function __construct(
        Action\Context $context,
        DataPersistorInterface $dataPersistor,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Pinpoint\ProductPersonalisation\Model\ImageTemplate $imageTemplate,
        \Pinpoint\ProductPersonalisation\Model\ImageRow $imageTemplateRow,
        \Pinpoint\ProductPersonalisation\Model\ImageRowRepository $imageTemplateRowRepository
    ) {
        $this->dataPersistor = $dataPersistor;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->imageTemplate = $imageTemplate;
        $this->imageTemplateRow = $imageTemplateRow;
        $this->imageTemplateRowRepository = $imageTemplateRowRepository;

        parent::__construct($context);
    }

    /**
     * Save action
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $resultJson = $this->resultJsonFactory->create();

        $imageTemplate = $this->imageTemplate;
        $imageTemplateRows = $this->imageTemplateRow;
        $data = $this->getRequest()->getParams();
        $deleteList = isset($data['deleteList']) ? $data['deleteList'] : [];
        unset($data['key'], $data['isAjax'], $data['form_key'], $data['deleteList']);
        $data = $this->parsePost($data);

        $template = isset($data['template']) ? $data['template'] : null;
        $rows = isset($data['rows']) ? $data['rows'] : null;


        if (isset($template['id']) && !empty($template['id'])) {
            $imageTemplate = $imageTemplate->load($template['id']);
        } else {
            unset($template['id']);
        }

        if (count(array_filter($template, 'strlen')) == count($template)) {
            if (!isset($template['template_image'])) {
                $template['template_image'] = '/media/pinpoint_pp_images/';
            }

            $imageTemplate->setData($template)->save();
        } else {
            return $resultJson->setData(['success' => false]);
        }

        if (count($rows)) {
            foreach ($rows as $row) {
                $templateRow = $imageTemplateRows;
                if (isset($row['id'])) {
                    $templateRow = $templateRow->load($row['id']);
                } else {
                    unset($row['id']);
                }

                $row['parent_template_id'] = $imageTemplate->getId();
                if (count(array_filter($row, 'strlen')) === count($row)
                    || (count($row) - count(array_filter($row, 'strlen'))) === 1 && $row['css'] === ''
                ) {
                    $templateRow->setData($row)->save();
                }
            }
        }

        if (isset($deleteList) && is_array($deleteList)) {
            foreach ($deleteList as $id) {
                $imageRow = $imageTemplateRows;
                $imageRow->load((int) $id);
                $this->imageTemplateRowRepository->delete($imageRow);
            }
        }

        return $resultJson->setData(['success' => true, 'location' => $this->getUrl('*/imagetemplates/templatelist')]);
    }

    /**
     * @param $data
     * @return array
     */
    protected function parsePost($data)
    {
        $output = [];
        foreach ($data as $parentKey => $array) {
            $output[$parentKey] = [];

            foreach ($array as $index => $field) {
                if ($parentKey == 'template') {
                    if (!isset($output[$parentKey])) {
                        $output[$parentKey] = [];
                    }

                    if (isset($field['value'])) {
                        $output[$parentKey][$field['name']] = $field['value'];
                    }
                }

                if ($parentKey == 'rows') {
                    $rowArr = [];
                    foreach ($field as $row) {
                        $rowArr[$row['name']] = $row['value'];
                    }

                    if (!isset($output[$parentKey])) {
                        $output[$parentKey] = [];
                    }
                    $output[$parentKey][] = $rowArr;
                }
            }
        }

        return $output;
    }
}
