<?php

namespace Pinpoint\ProductPersonalisation\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;

/**
 * Class Data
 * @package Pinpoint\ProductPersonalisation\Helper
 */
class Data extends AbstractHelper
{
    /**
     * @var \Magento\Framework\Registry
     */
    protected $registry;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageRow
     */
    protected $imageRow;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageTemplate
     */
    protected $imageTemplate;

    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageRow
     */
    public $row;

    protected $session;

    /**
     * Data constructor.
     * @param Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Pinpoint\ProductPersonalisation\Model\ImageTemplate $imageTemplate
     * @param \Pinpoint\ProductPersonalisation\Model\ImageRow $imageRow
     * @param \Magento\Catalog\Api\ProductRepositoryInterface $productRepository
     */
    public function __construct(
        Context $context,
        \Magento\Framework\Registry $registry,
        \Pinpoint\ProductPersonalisation\Model\ImageTemplate $imageTemplate,
        \Pinpoint\ProductPersonalisation\Model\ImageRow $imageRow,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Customer\Model\Session $session
    ) {
        $this->registry = $registry;
        $this->imageRow = $imageRow;
        $this->imageTemplate = $imageTemplate;
        $this->productRepository = $productRepository;
        $this->session = $session;

        parent::__construct($context);
    }

    /**
     * @param $templateId
     * @return int
     */
    public function setCurrentTemplate($templateId)
    {
        $this->registry->unregister('ppp_current_template_id');
        $this->registry->register('ppp_current_template_id', $templateId);
        $this->session->setData('ppp_current_template_id', $templateId);

        return $templateId;
    }

    public function setFilteredSku($sku)
    {
        $this->session->setData('ppp_filtered_sku', $sku);
    }

    public function getFilteredSku()
    {
        return $this->session->getData('ppp_filtered_sku');
    }

    public function setUsingGenerated()
    {
        $this->registry->unregister('using_generated_template');
        $this->registry->register('using_generated_template', true);
        $this->session->setData('using_generated_template', true);
    }

    public function isUsingGenerated()
    {
        return $this->session->getData('using_generated_template') ? $this->session->getData('ppp_current_template_id') : false;
    }

    /**
     * @param array|object $option
     * @param null|int $tid
     * @return string|null
     */
    public function getCurrentTemplateRowName($option, $tid = null)
    {
        $templateId = $tid ? $tid : $this->registry->registry('ppp_current_template_id');
        if ($templateId) {
            $optionTitle = null;

            if (gettype($option) == 'array') {
                $optionTitle = $option['label'];
            } else {
                $optionTitle = $option->getTitle();
            }
            if (substr($optionTitle, 0, 4) == 'Line') {
                $rows = $this->imageRow->getCollection()->addFieldToFilter('parent_template_id', $templateId)->getItems();

                $line = (int) explode(" ", $optionTitle)[1] - 1;

                if (isset(array_keys($rows)[$line])) {
                    $this->row = $rows[array_keys($rows)[$line]];
                    return $rows[array_keys($rows)[$line]]->getData('row_label');
                }
            }
        }

        return null;
    }

    /**
     * @param $option
     * @param null $tid
     * @return mixed|null
     */
    public function getCurrentTemplateRowLength($option, $tid = null)
    {
        $templateId = $tid ? $tid : $this->registry->registry('ppp_current_template_id');
        if ($templateId) {
            $optionTitle = null;

            if (gettype($option) == 'array') {
                $optionTitle = $option['label'];
            } else {
                $optionTitle = $option->getTitle();
            }
            if (substr($optionTitle, 0, 4) == 'Line') {
                $rows = $this->imageRow->getCollection()->addFieldToFilter('parent_template_id', $templateId)->getItems();

                $line = (int) explode(" ", $optionTitle)[1] - 1;

                if (isset(array_keys($rows)[$line])) {
                    $this->row = $rows[array_keys($rows)[$line]];
                    return $rows[array_keys($rows)[$line]]->getData('character_limit');
                }
            }
        }

        return null;
    }

    /**
     * @param $sku
     * @param $options
     * @return \Magento\Framework\DataObject
     */
    public function getCurrentTemplate($sku, $options)
    {
        $template = $this->imageTemplate;

        if ($this->isUsingGenerated()) {
            return $template->getCollection()
                ->addFieldToFilter(
                    'id',
                    $this->session->getData('ppp_current_template_id')
                )
                ->getFirstItem()
                ;
        }

        $lineTotal = 0;

        foreach ($options as $option) {
            if ($option->getData('type') == 'field') {
                if (substr($option->getData('title'), 0, 4) == 'Line') {
                    $lineTotal++;
                }
            }
        }

        return $template->getCollection()
            ->addFieldToFilter('template_id', $sku)
            ->addFieldToFilter('row_count', $lineTotal)
            ->getFirstItem()
        ;
    }

    /**
     * @param $sku
     * @return mixed
     */
    public function getProductUrlFromSku($sku)
    {
        $product = $this->productRepository->get($sku);

        return $product->getUrlModel()->getUrl($product);
    }

}
