<?php
namespace Pinpoint\ProductPersonalisation\Model\ImageTemplate;

use Pinpoint\ProductPersonalisation\Model\ResourceModel\ImageTemplate\CollectionFactory;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\App\Filesystem\DirectoryList;

class DataProvider extends \Magento\Ui\DataProvider\AbstractDataProvider
{
    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ResourceModel\ImageTemplate\Collection
     */
    protected $collection;

    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @var \Magento\Framework\Filesystem
     */
    protected $fileSystem;

    /**
     * @var array
     */
    protected $loadedData;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * DataProvider constructor.
     * @param string $name
     * @param string $primaryFieldName
     * @param string $requestFieldName
     * @param CollectionFactory $collectionFactory
     * @param DataPersistorInterface $dataPersistor
     * @param \Magento\Framework\Filesystem $fileSystem
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param array $meta
     * @param array $data
     */
    public function __construct(
        $name,
        $primaryFieldName,
        $requestFieldName,
        CollectionFactory $collectionFactory,
        DataPersistorInterface $dataPersistor,
        \Magento\Framework\Filesystem $fileSystem,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        array $meta = [],
        array $data = []
    ) {
        $this->collection = $collectionFactory->create();
        $this->dataPersistor = $dataPersistor;
        $this->fileSystem = $fileSystem;
        $this->storeManager = $storeManager;

        parent::__construct($name, $primaryFieldName, $requestFieldName, $meta, $data);
        
        $this->meta = $this->prepareMeta($this->meta);
    }

    /**
     * Prepares Meta
     *
     * @param array $meta
     * @return array
     */
    public function prepareMeta(array $meta)
    {
        return $meta;
    }

    /**
     * Get data
     *
     * @return array
     */
    public function getData()
    {
        if (isset($this->loadedData)) {
            return $this->loadedData;
        }

        $items = $this->collection->getItems();
        foreach ($items as $item) {
            $filePath = $this->fileSystem->getDirectoryRead(DirectoryList::ROOT)->getAbsolutePath();
            $templateImage = $item->getData('template_image');

            $mediaUrl = $this->storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA);
            if (strpos($mediaUrl, '/pub/') === false) {
                $item->setData('template_image', str_replace('pub/', '', $item->getData('template_image')));
            }

            $item->setData('template_image', str_replace("//", "/", $item->getData('template_image')));

            $fileExplode = explode('.', $item->getData('template_image'));
            $fileType = str_replace('JPG', 'JPEG', strtoupper(end($fileExplode)));

            if (basename($item->getData('template_image')) !== 'pinpoint_pp_images' && $item->getData('template_image') !== '') {
                $item->setData('template_image', [[
                    'url' => $item->getData('template_image'),
                    'file' => $item->getData('template_image'),
                    'name' => basename($item->getData('template_image')),
                    'type' => image_type_to_mime_type(constant('IMAGETYPE_' . $fileType)),
                    'size' => filesize(
                        $this->fileSystem->getDirectoryRead(DirectoryList::PUB)->getAbsolutePath()
                        . str_replace("pub/", '', $item->getData('template_image'))
                    )
                ]]);
            } else {
                $item->unsetData('template_image');
            }

            $this->loadedData[$item->getId()] = $item->getData();
        }

        $data = $this->dataPersistor->get('pinpoint_productpersonalisation_imagetemplate');
        if (!empty($data)) {
            $item = $this->collection->getNewEmptyItem();
            $item->setData($data);
            $this->loadedData[$item->getId()] = $item->getData();
            $this->dataPersistor->clear('pinpoint_productpersonalisation_imagetemplate');
        }

        return $this->loadedData;
    }
}
