<?php

namespace Pinpoint\ProductPersonalisation\Model\ImageTemplate;

use Magento\Framework\App\Filesystem\DirectoryList;

class Preview extends \Magento\Framework\Model\AbstractModel
{
    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $product;

    /**
     * @var \Magento\Framework\Filesystem
     */
    protected $fileSystem;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageTemplate
     */
    protected $imageTemplate;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageRow
     */
    protected $imageRow;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Catalog\Model\ProductFactory $product,
        \Magento\Framework\Filesystem $fileSystem,
        \Pinpoint\ProductPersonalisation\Model\ImageTemplate $imageTemplate,
        \Pinpoint\ProductPersonalisation\Model\ImageRow $imageRow,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->product = $product;
        $this->fileSystem = $fileSystem;
        $this->imageTemplate = $imageTemplate;
        $this->imageRow = $imageRow;

        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    public function drawGuidelinePreview(\Magento\Framework\App\RequestInterface $request)
    {
        $product = $this->product->create();
        $result = [];

      if ($productSku = $request->getParam('sku')) {
            $product = $product->loadByAttribute('sku', $productSku);
            $options = $this->extractOptions($request->getParam('options'));

            $result['image'] = '';

            if ($product->getId()) {
                $basePath = $this->fileSystem->getDirectoryRead(DirectoryList::ROOT)->getAbsolutePath();
                $data = $this->getTemplateByRowCount($productSku, $this->getLineTotal($options));

                $im = new \Imagick($basePath . $data['template']['template_image']);

                foreach ($options as $key => $option) {
                    if (substr($key, 0, 4) == 'line') {
                        $draw = new \ImagickDraw();
                        $lineNumber = explode("_", $key)[1];

                        $row = $this->getTemplateRowArrayKey($lineNumber, $data['rows']);
                        $row = $data['rows'][$row];

                        $distanceToCanvas = explode(",", $data['template']['distance_to_canvas']);

                        $strokeColor = new \ImagickPixel("#ff0000");

                        $draw->setStrokeColor($strokeColor);
                        $draw->setStrokeOpacity(1);
                        $draw->setStrokeWidth(1);
                        $draw->setFillOpacity(0);

                        $draw->rectangle(
                            ($distanceToCanvas[1] + $row['left']) - 1 - ($row['width'] - $row['left'] / 2),
                            ($distanceToCanvas[0] + $row['top']) - 1 - $row['height'],
                            ($distanceToCanvas[1] + $row['width']) + 1 + ($row['left']),
                            ($distanceToCanvas[0] + $row['top']) + 1
                        );

                        $im->drawImage($draw);
                    }
                }

                $result['image'] = base64_encode($im->getImageBlob());
            }
        }

        return $result;
    }

    /**
     * @param $index
     * @param $rows
     * @return int
     */
    public function getTemplateRowArrayKey($index, $rows)
    {
        return (int)array_keys($rows)[$index-1];
    }

    /**
     * @param $sku
     * @param $rowCount
     * @return array|null
     */
    public function getTemplateByRowCount($sku, $rowCount)
    {
        $templates = $this->imageTemplate->getCollection()->addFieldToFilter('template_id', $sku);

        foreach ($templates->getItems() as $template) {

            if ($template->getData('row_count') == $rowCount) {
                $rows = $this->imageRow->getCollection()->addFieldToFilter('parent_template_id', $template->getId());
                return ['template' => $template, 'rows' => $rows->getItems()];
            }
        }

        return null;
    }

    /**
     * @param $options
     * @return int
     */
    protected function getLineTotal($options)
    {
        $count = 0;
        foreach ($options as $key => $value) {
            if (substr($key, 0, 5) == "line_") {
                $count++;
            }
        }

        return $count;
    }

    /**
     * @param $post
     * @return array
     */
    protected function extractOptions($post)
    {
        $options = [];

        foreach (\json_decode($post, true) as $item) {
            if (substr($item['name'], 0, 7) == 'options' && isset($item['label'])) {
                $options[str_replace(" ", "_", strtolower($item['label']))] = $item['value'];
            }
        }

        return $options;
    }

}