<?php

namespace Pinpoint\ProductPersonalisation\Model\Pdf;

/**
 * Class Generation
 * @package Pinpoint\ProductPersonalisation\Model\Pdf
 */
class Generation extends \Magento\Framework\Model\AbstractModel
{
    /**
     * @var int
     */
    protected $pageHeight = 80;

    /**
     * @var int
     */
    protected $pageWidth = 80;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageTemplate
     */
    protected $imageTemplate;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageRow
     */
    protected $imageRow;

    /**
     * @var \Pinpoint\Feefo\Helper\Data
     */
    protected $personalisationHelper;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\Font
     */
    protected $productFont;

    /**
     * @var \Magento\Framework\Filesystem
     */
    protected $fileSystem;

    const TEXT_ALIGN_LEFT = 'left';

    const TEXT_ALIGN_RIGHT = 'right';

    const TEXT_ALIGN_CENTER = 'center';

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Pinpoint\ProductPersonalisation\Model\ImageTemplate $imageTemplate,
        \Pinpoint\ProductPersonalisation\Model\ImageRow $imageRow,
        \Pinpoint\ProductPersonalisation\Helper\Data $personalisationHelper,
        \Pinpoint\ProductPersonalisation\Model\Font $productFont,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->imageTemplate = $imageTemplate;
        $this->imageRow = $imageRow;
        $this->personalisationHelper = $personalisationHelper;
        $this->productFont = $productFont;
        $this->fileSystem = $filesystem;

        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * @param $item
     * @return \Mpdf\Mpdf
     */
    public function generateMachinePrint($item)
    {
        $options = $item->getProductOptions()['options'];

        $generatedId = false;
        if ($additionOptions = $item->getProductOptionByCode('additional_options')) {
            if (@end($additionOptions)['label'] === 'used_template') {
                if ($additionOptions[0]['label'] === 'Font') {
                    $additionOptions[0]['label'] = 'Choose Font';
                }
                $generatedId = @end($additionOptions)['value'];

                unset($additionOptions[count($additionOptions)-1]);
                $options = $additionOptions;
            }
        }

        if ($generatedId) {
            $imageTemplate = $this->imageTemplate
                ->getCollection()
                ->addFieldToFilter('id', $generatedId)
                ->addFieldToFilter('row_count', $this->getImageRowCount($options))
                ->getFirstItem();
        } else {
            $imageTemplate = $this->imageTemplate
                ->getCollection()
                ->addFieldToFilter('template_id', $item->getSku())
                ->addFieldToFilter('row_count', $this->getImageRowCount($options))
                ->getFirstItem();
        }

        $defaultConfig = (new \Mpdf\Config\ConfigVariables())->getDefaults();
        $fontDirs = $defaultConfig['fontDir'];

        $defaultFontConfig = (new \Mpdf\Config\FontVariables())->getDefaults();
        $fontData = $defaultFontConfig['fontdata'];
        $font = $this->productFont->load($this->getChosenFont(['options' => $options]), 'font_name');

        $mpdf = new \Mpdf\Mpdf([
            'mode' => 'UTF-8',
            'format' => [$imageTemplate->getData('outer_width'), $imageTemplate->getData('outer_canvas')],
            'fontDir' => array_merge($fontDirs, [
                $this->fileSystem
                    ->getDirectoryRead(\Magento\Framework\App\Filesystem\DirectoryList::MEDIA)
                    ->getAbsolutePath() . \Pinpoint\ProductPersonalisation\Model\Font::MEDIA_DIR,
            ]),
            'tempDir' => $this->fileSystem
                    ->getDirectoryRead(\Magento\Framework\App\Filesystem\DirectoryList::MEDIA)
                    ->getAbsolutePath() . 'mpdf_temp',
            'fontdata' => $fontData + [
                    'pdffont' => [
                        'R' => $font->getData('font_file'),
                    ]
                ],
            'default_font' => 'pdffont',
            'margin_left'=> 0,
            'margin_right'=> 0,
            'margin_top'=> 0,
            'margin_bottom'=> 0,
            'margin_header'=> 0,
            'margin_footer'=> 0
        ]);


        if ($imageTemplate) {
            $multiPreviewOptions = $item->getProductOptionByCode('additional_options');
            $pageCount = count($multiPreviewOptions) ? 2 : 1;

            if ($pageCount === 2 && $generatedId && !(int)$imageTemplate->getData('multiple_previews')) {
                $pageCount = 1;
            } else {
                $options = $item->getProductOptions()['options'];
            }

            for ($page = 0; $page < $pageCount; $page++) {
                $pageHtml = '';
                if ($page > 0) {
                    $options = $multiPreviewOptions;
                    $mpdf->AddPage();
                }

                foreach ($options as $option) {
                    if ($option['label'] == 'Choose Font') {
                        $option['option_type'] = 'select';
                    } else {
                        $option['option_type'] = 'field';
                    }


                    if ($option['option_type'] == 'field') {
                        $optionLabel = $this->personalisationHelper->getCurrentTemplateRowName($option, $imageTemplate->getId());
                        $row = $this->imageRow
                            ->getCollection()
                            ->addFieldToFilter('parent_template_id', $imageTemplate->getId())
                            ->addFieldToFilter('row_label', $optionLabel)
                            ->getFirstItem();

                        $line_height = '';
                        $att = '';

                        $pageHtml .= '<div style="position:absolute; left:' . $row->getData('left') . 'mm; top:' . $row->getData('top') . 'mm; width:' . $row->getData('width') . 'mm; height:' . $row->getData('height') . 'mm;"><div style="position:relative; width:100%; height:' . $row->getData('height') . 'mm;"><div style="position:absolute; width:100%;' . $line_height . $att . ' line-height: 1em;font-size:' . $row->getData('font_size') . 'px; text-align:' . $row->getData('text_alignment') . ';"><span>' . html_entity_decode($option['value'], ENT_QUOTES) . '</span></div></div></div>';

                        continue;
                    }
                }

                $mpdf->WriteHTML($pageHtml);
            }
        }

        $mpdf->SetDisplayMode('fullpage');

        return $mpdf;
    }


    /**
     * @param $options
     * @return int
     */
    protected function getImageRowCount($options)
    {
        $count = 0;
        foreach ($options as $option) {
            if (substr($option['label'], 0, 5) == 'Line ') {
                $count++;
            }
        }

        return $count;
    }

    /**
     * @param $options
     * @return null|string
     */
    protected function getChosenFont($options)
    {
        foreach ($options['options'] as $option) {
            if ($option['label'] == 'Choose Font') {
                return $option['value'];
            }
        }

        return null;
    }
}
