<?php

namespace Pinpoint\ProductPersonalisation\Observer\Checkout\Cart;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer;
use Magento\Checkout\Model\Cart\Interceptor;
use Magento\Framework\App\Filesystem\DirectoryList;

/**
 * Class UpdateItemsAfter
 * @package Pinpoint\ProductPersonalisation\Observer\Checkout\Cart
 */
class UpdateItemsAfter implements ObserverInterface
{
    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $request;

    /**
     * @var \Magento\Framework\Filesystem
     */
    protected $fileSystem;

    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    protected $messageManager;

    /**
     * UpdateItemsAfter constructor.
     * @param \Magento\Framework\App\RequestInterface $request
     * @param \Magento\Framework\Filesystem $fileSystem
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     */
    public function __construct(
        \Magento\Framework\App\RequestInterface $request,
        \Magento\Framework\Filesystem $fileSystem,
        \Magento\Framework\Message\ManagerInterface $messageManager
    ) {
        $this->request = $request;
        $this->fileSystem = $fileSystem;
        $this->messageManager = $messageManager;
    }

    /**
     * Execute Observer
     *
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        /**
         * @var Interceptor $cartF
         */
        $cart = $observer->getData('cart');
        $info = $observer->getData('info');

        $items = $cart->getQuote()->getItems();
        foreach ($items as $item) {
            $updatedOptions = $info[$item->getId()];
            if ($cartFiles = $this->request->getFiles('cart')) {
                if (isset($cartFiles[$item->getId()])) {
                    $updatedOptions = array_merge($cartFiles[$item->getId()], $updatedOptions);
                }
            }

            if (!$updatedOptions) continue;
            foreach ($updatedOptions as $optionKey => $value) {
                if ($option = $item->getOptionByCode($optionKey)) {
                    $option = $option->getData();

                    if ($this->isLogoEngraving($optionKey, $item->getId())) {
                        list($width, $height) = getimagesize($_FILES['cart']['tmp_name'][$item->getId()][$optionKey]);

                        $optionData = [
                            'type' => $_FILES['cart']['type'][$item->getId()][$optionKey],
                            'title' => $_FILES['cart']['name'][$item->getId()][$optionKey],
                            'size' => $_FILES['cart']['size'][$item->getId()][$optionKey],
                            'width' => $width,
                            'height' => $height,
                            'quote_path' => null,
                            'order_path' => null,
                            'fullpath' => null,
                            'secret_key' => json_decode($option['value'], true)['secret_key']
                        ];

                        $mediaDirectory = $this->fileSystem->getDirectoryRead(DirectoryList::MEDIA);

                        $fileName = md5(rand(1000,9000) . time() . $optionData['title']) . '.' . explode(".", $optionData['title'])[1];

                        $fileUploaded = move_uploaded_file(
                            $_FILES['cart']['tmp_name'][$item->getId()][$optionKey],
                            $mediaDirectory->getAbsolutePath() . DIRECTORY_SEPARATOR . 'custom_options' . DIRECTORY_SEPARATOR . 'quote' . DIRECTORY_SEPARATOR . $fileName
                        );

                        if ($fileUploaded) {
                            $optionData['order_path'] = $optionData['quote_path'] = 'custom_options' . DIRECTORY_SEPARATOR . 'quote' . DIRECTORY_SEPARATOR . $fileName;
                            $optionData['fullpath'] = $mediaDirectory->getAbsolutePath() . DIRECTORY_SEPARATOR . 'custom_options' . DIRECTORY_SEPARATOR . 'quote' . DIRECTORY_SEPARATOR . $fileName;
                            $value = \json_encode($optionData);
                        } else {
                            $this->messageManager->addError(__('Failed to uploaded Logo Engraving image.'));
                            continue;
                        }
                    }

                    $option['value'] = $value;
                    $item->addOption($option);
                    $item->saveItemOptions();
                }
            }
        }
    }

    protected function isLogoEngraving($optionKey, $itemId)
    {
        if (isset($_FILES['cart']) && isset($_FILES['cart']['name'][$itemId])) {
            if (array_keys($_FILES['cart']['name'][$itemId])[0] == $optionKey) {
                return true;
            }
        }

        return false;
    }
}
