<?php

namespace Pinpoint\ProductPersonalisation\Observer;

use Magento\Framework\DataObject;
use Magento\Framework\Event\Observer as EventObserver;
use Magento\Framework\Event\ObserverInterface;

/**
 * Class CheckoutCartProductAddAfterObserver
 * @package Pinpoint\ProductPersonalisation\Observer
 */
class CheckoutCartProductAddAfterObserver implements ObserverInterface
{
    /**
     * @var \Magento\Framework\View\LayoutInterface
     */
    protected $_layout;
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $_request;

    /**
     * @var \Magento\Framework\Serialize\Serializer\Json
     */
    protected $serializer;

    protected $session;

    public function __construct(
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\View\LayoutInterface $layout,
        \Magento\Framework\App\RequestInterface $request,
        \Magento\Framework\Serialize\Serializer\Json $serializer,
        \Magento\Customer\Model\Session $session
    ) {
        $this->_layout = $layout;
        $this->_storeManager = $storeManager;
        $this->_request = $request;
        $this->serializer = $serializer;
        $this->session = $session;
    }
    /**
     * Add order information into GA block to render on checkout success pages
     *
     * @param EventObserver $observer
     * @return void
     */
    public function execute(EventObserver $observer)
    {
        if (
            !$this->_request->getParam('multiple_previews')
            && !$this->_request->getParam('no_personalisation')
            && !$this->_request->getParam('using_generated_fields')
        ) {
            \Magento\Framework\App\ObjectManager::getInstance()->get(\Psr\Log\LoggerInterface::class)->debug('jumping out');
            return;
        }

        /* @var \Magento\Quote\Model\Quote\Item $item */
        $item = $observer->getQuoteItem();

        $additionalOptions = array();

        if ($additionalOption = $item->getOptionByCode('additional_options')) {
            $additionalOptions = (array) $this->serializer->unserialize($additionalOption->getValue());
        }

        $post = $this->_request->getParam('multi_preview_options');
        if (is_array($post)) {
            foreach ($post as $key => $value) {
                if ($key == '' || $value == '') {
                    continue;
                }
                $additionalOptions[] = [
                    'label' => $key,
                    'value' => $value
                ];
            }
        }

        $options = $this->_request->getParam('options');
        if (is_array($options)) {
            foreach ($options as $key => $value) {
                if ($key == '' || $value == '') {
                    continue;
                }

                if (strpos($key , 'line_') !== false) {
                    $label = explode('_', $key);
                    $additionalOptions[] = [
                        'label' => ucfirst($label[0]) . ' ' . $label[1],
                        'value' => $value
                    ];
                }

                if ($key == 'font') {
                    $additionalOptions[] = [
                        'label' => __('Font'),
                        'value' => $value
                    ];
                }
            }

            $additionalOptions[] = [
                'label' => 'used_template',
                'value' => $this->session->getData('ppp_current_template_id')
            ];
        }

        if ($this->_request->getParam('no_personalisation')) {
            $additionalOptions[] = [
                'label' => __("Personalisation Required"),
                'value' => __('No')
            ];
        }

        if (count($additionalOptions) > 0) {
            $item->addOption([
                'product_id' => $item->getProductId(),
                'code' => 'additional_options',
                'value' => $this->serializer->serialize($additionalOptions)
            ]);
        }
    }
}
