<?php

namespace Pinpoint\ProductPersonalisation\Setup;

use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Eav\Setup\EavSetup;
use Magento\Eav\Setup\EavSetupFactory;

/**
 * Class UpgradeData
 * @package Pinpoint\ProductPersonalisation\Setup
 */
class UpgradeData extends \Pinpoint\Core\Model\Setup\PinpointUpgrade
{
    /**
     * @var EavSetupFactory
     */
    protected $eavSetupFactory;

    /**
     * UpgradeData constructor.
     * @param EavSetupFactory $eavSetupFactory
     * @param \Magento\Framework\Module\Dir\Reader $moduleReader
     * @param \Magento\Framework\Xml\Parser $reader
     */
    public function __construct(
        EavSetupFactory $eavSetupFactory,
        \Magento\Framework\Module\Dir\Reader $moduleReader,
        \Magento\Framework\Xml\Parser $reader
    ) {
        $this->eavSetupFactory = $eavSetupFactory;


        parent::__construct($moduleReader, $reader);
    }

    /**
     * @param ModuleDataSetupInterface $setup
     */
    protected function upgrade_0_0_3(ModuleDataSetupInterface $setup)
    {
        /** @var EavSetup $eavSetup */
        $eavSetup = $this->eavSetupFactory->create(['setup' => $setup]);

        if (!$eavSetup->getAttributeId(\Magento\Catalog\Model\Product::ENTITY, 'pinpoint_pp_is_personalised')) {
            $eavSetup->addAttribute(
                \Magento\Catalog\Model\Product::ENTITY,
                'pinpoint_pp_is_personalised',
                [
                    'type' => 'int',
                    'label' => 'Is Personalised?',
                    'input' => 'select',
                    'required' => false,
                    'sort_order' => 9999,
                    'source' => \Magento\Eav\Model\Entity\Attribute\Source\Boolean::class,
                    'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                    'used_in_product_listing' => true,
                    'visible_on_front' => false,
                    'visible' => true,
                    'user_defined' => false,
                    'default' => 1,
                    'searchable' => false,
                    'filterable' => false,
                    'comparable' => false,
                    'unique' => false
                ]
            );
        }
    }

    /**
     * @param ModuleDataSetupInterface $setup
     */
    protected function upgrade_0_0_5(ModuleDataSetupInterface $setup)
    {
        /** @var EavSetup $eavSetup */
        $eavSetup = $this->eavSetupFactory->create(['setup' => $setup]);

        if (!$eavSetup->getAttributeId(\Magento\Catalog\Model\Product::ENTITY, 'ppp_has_personalised_opt')) {
            $eavSetup->addAttribute(
                \Magento\Catalog\Model\Product::ENTITY,
                'ppp_has_personalised_opt',
                [
                    'type' => 'int',
                    'label' => 'Has personalisation options?',
                    'input' => 'select',
                    'required' => false,
                    'sort_order' => 9999,
                    'source' => \Magento\Eav\Model\Entity\Attribute\Source\Boolean::class,
                    'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                    'used_in_product_listing' => true,
                    'visible_on_front' => false,
                    'visible' => true,
                    'user_defined' => false,
                    'default' => 1,
                    'searchable' => false,
                    'filterable' => false,
                    'comparable' => false,
                    'unique' => false
                ]
            );
        }
    }

    /**
     * @param ModuleDataSetupInterface $setup
     */
    protected function upgrade_0_0_9(ModuleDataSetupInterface $setup)
    {
        /** @var EavSetup $eavSetup */
        $eavSetup = $this->eavSetupFactory->create(['setup' => $setup]);
        $eavSetup->removeAttribute(\Magento\Catalog\Model\Product::ENTITY, 'ppp_item_type');
        if (!$eavSetup->getAttributeId(\Magento\Catalog\Model\Product::ENTITY, 'ppp_item_type')) {
            $eavSetup->addAttribute(
                \Magento\Catalog\Model\Product::ENTITY,
                'ppp_item_type',
                [
                    'type' => 'varchar',
                    'label' => 'Item Type',
                    'input' => 'select',
                    'required' => false,
                    'sort_order' => 9999,
                    'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                    'used_in_product_listing' => true,
                    'visible_on_front' => false,
                    'visible' => true,
                    'user_defined' => false,
                    'default' => 1,
                    'searchable' => false,
                    'filterable' => false,
                    'comparable' => false,
                    'unique' => false
                ]
            );
        }
    }
}