<?php

namespace Pinpoint\ProductPersonalisation\Setup;

use Magento\Framework\DB\Ddl\Table;
use Magento\Framework\Setup\UpgradeSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

/**
 * @codeCoverageIgnore
 */
class UpgradeSchema implements UpgradeSchemaInterface
{
    /**
     * {@inheritdoc}
     */
    public function upgrade(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $setup->startSetup();

        if (version_compare($context->getVersion(), '0.0.2', '<')) {
            $this->createFontTable($setup);
        }

        if (version_compare($context->getVersion(), '0.0.4', '<')) {
            $this->createImageTemplateTable($setup);
            $this->createImageTemplateRowTable($setup);
        }

        if (version_compare($context->getVersion(), '0.0.6', '<')) {
            $this->addImageTemplateOrientation($setup);
        }

        if (version_compare($context->getVersion(), '0.0.7', '<')) {
            $this->addMultiplePreviewColumn($setup);
        }

        if (version_compare($context->getVersion(), '0.0.8', '<')) {
            $this->addCssAndColourColumn($setup);
        }

        $setup->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function createFontTable(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $installer->startSetup();

        $tableName = $installer->getTable('pinpoint_product_personalisation_fonts');
        // Check if the table already exists
        if ($installer->getConnection()->isTableExists($tableName) != true) {
            $table = $installer->getConnection()
                ->newTable($tableName)
                ->addColumn(
                    'id',
                    Table::TYPE_INTEGER,
                    null,
                    [
                        'identity' => true,
                        'unsigned' => true,
                        'nullable' => false,
                        'primary' => true
                    ],
                    'ID'
                )
                ->addColumn(
                    'font_name',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Font Name'
                )
                ->addColumn(
                    'font_file',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Font File Name'
                )
                ->addColumn(
                    'enabled',
                    Table::TYPE_SMALLINT,
                    null,
                    ['nullable' => false, 'default' => 1],
                    'Enabled'
                )
                ->setComment('Pinpoint PP Fonts')
                ->setOption('type', 'InnoDB')
                ->setOption('charset', 'utf8');
            $installer->getConnection()->createTable($table);
        }

        $installer->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function createImageTemplateTable(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $installer->startSetup();

        $tableName = $installer->getTable('pinpoint_product_personalisation_image_template');
        // Check if the table already exists
        if ($installer->getConnection()->isTableExists($tableName) != true) {
            $table = $installer->getConnection()
                ->newTable($tableName)
                ->addColumn(
                    'id',
                    Table::TYPE_INTEGER,
                    null,
                    [
                        'identity' => true,
                        'unsigned' => true,
                        'nullable' => false,
                        'primary' => true
                    ],
                    'ID'
                )
                ->addColumn(
                    'canvas_size',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Canvas Size'
                )
                ->addColumn(
                    'template_id',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Template ID (Product SKU)'
                )
                ->addColumn(
                    'distance_to_canvas',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Distance To Canvas'
                )
                ->addColumn(
                    'outer_width',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Outer Width'
                )
                ->addColumn(
                    'outer_canvas',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Outer Canvas'
                )
                ->addColumn(
                    'row_count',
                    Table::TYPE_INTEGER,
                    null,
                    ['nullable' => false],
                    'Template Row Count'
                )
                ->addColumn(
                    'template_image',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Template Image'
                )
                ->addColumn(
                    'enabled',
                    Table::TYPE_SMALLINT,
                    null,
                    ['nullable' => false, 'default' => 1],
                    'Enabled'
                )
                ->setComment('Pinpoint PP Image Templates')
                ->setOption('type', 'InnoDB')
                ->setOption('charset', 'utf8');
            $installer->getConnection()->createTable($table);
        }

        $installer->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function createImageTemplateRowTable(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $installer->startSetup();

        $tableName = $installer->getTable('pinpoint_product_personalisation_image_template_row');
        // Check if the table already exists
        if ($installer->getConnection()->isTableExists($tableName) != true) {
            $table = $installer->getConnection()
                ->newTable($tableName)
                ->addColumn(
                    'id',
                    Table::TYPE_INTEGER,
                    null,
                    [
                        'identity' => true,
                        'unsigned' => true,
                        'nullable' => false,
                        'primary' => true
                    ],
                    'ID'
                )
                ->addColumn(
                    'row_label',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Row Label'
                )
                ->addColumn(
                    'font_size',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Font Size'
                )
                ->addColumn(
                    'character_limit',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Row Character Limit'
                )
                ->addColumn(
                    'width',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Width'
                )
                ->addColumn(
                    'height',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Width'
                )
                ->addColumn(
                    'left',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Distance from left'
                )
                ->addColumn(
                    'top',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Distance from top'
                )
                ->addColumn(
                    'text_alignment',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Text Alignment'
                )
                ->addColumn(
                    'parent_template_id',
                    Table::TYPE_SMALLINT,
                    null,
                    ['nullable' => false,],
                    'Parent Template Id'
                )
                ->setComment('Pinpoint PP Image Template Row')
                ->setOption('type', 'InnoDB')
                ->setOption('charset', 'utf8');
            $installer->getConnection()->createTable($table);
        }

        $installer->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function addImageTemplateOrientation(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $installer->startSetup();
        $eavTable = $installer->getTable('pinpoint_product_personalisation_image_template');

        $columns = [
            'orientation' => [
                'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                'nullable' => false,
                'comment' => 'Orientation',
            ],

        ];

        $connection = $installer->getConnection();
        foreach ($columns as $name => $definition) {
            $connection->addColumn($eavTable, $name, $definition);
        }

        $installer->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function addMultiplePreviewColumn(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $installer->startSetup();
        $eavTable = $installer->getTable('pinpoint_product_personalisation_image_template');

        $columns = [
            'multiple_previews' => [
                'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                'nullable' => false,
                'comment' => 'Multiple Previews',
            ],

        ];

        $connection = $installer->getConnection();
        foreach ($columns as $name => $definition) {
            $connection->addColumn($eavTable, $name, $definition);
        }

        $installer->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function addCssAndColourColumn(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $installer->startSetup();
        $eavTable = $installer->getTable('pinpoint_product_personalisation_image_template_row');

        $columns = [
            'css' => [
                'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                'nullable' => true,
                'comment' => 'Line CSS',
            ],
            'colour' => [
                'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                'nullable' => true,
                'comment' => 'Line Colour',
            ]
        ];

        $connection = $installer->getConnection();
        foreach ($columns as $name => $definition) {
            $connection->addColumn($eavTable, $name, $definition);
        }

        $installer->endSetup();
    }
}
